# Provices some native objc wrappers to ease interoperability.

import backend, times

# Define some objc types to have them available in nimrod.
type
  NSString* {.importc: "NSString*", header: "<Foundation/Foundation.h>",
    final.} = object

  NSDate* {.importc: "NSDate*", header: "<Foundation/Foundation.h>",
    final.} = object

  NSMutableArray* {.importc: "NSMutableArray*",
    header: "<Foundation/Foundation.h>", final.} = object

  NRTodo* {.importc: "NRTodo*", header: "NRTodo.h", final.} = object

  NRPagedParams* {.importc: "NRPagedParams*",
    header: "NRPagedParams.h", final.} = object


proc newNSString*(text : string): NSString {.NoStackFrame, inline.} =
  ## Returns a new NSString object from a nimrod string.
  let ctext = cstring(text)
  {.emit: """return [NSString stringWithFormat:@"%s", `ctext`];""".}

proc newNSDate*(time: TTime): NSDate {.NoStackFrame, inline.} =
  ## Returns a new NSDate object from a nimrod unix epoc time.
  {.emit: """return [NSDate dateWithTimeIntervalSince1970:`time`];""".}

## Returns the utf8 cString for the NSString object.
proc getCString*(self: NSString): cstring {.importobjc: "UTF8String", nodecl.}

proc newNSMutableArray*(capacity: int): NSMutableArray {.
    NoStackFrame, inline.} =
  ## Returns a new mutable array with the specified capacity.
  {.emit: """return [NSMutableArray arrayWithCapacity:`capacity`];""".}

## Adds an object to a mutable array.
proc addObject*(self: NSMutableArray; todo: NRTodo) {.
  importobjc: "addObject", nodecl.}

## Returns a new uninitialized NRTodo object, use update on it.
proc newNRTodo*: NRTodo {.importobjc: "[NRTodo new] autorelease", nodecl.}

proc update*(self: NRTodo; todo: TTodo) =
  ## Updates a NRTodo object with values from its nimrod counterpart.
  let todoId = todo.getId
  {.emit: """self.id_ = `todoId`;""".}
  let text = newNSString(todo.text)
  {.emit: """self.text = `text`;""".}
  let priority = todo.priority
  {.emit: """self.priority = `priority`;""".}
  let isDone = todo.isDone
  {.emit: """self.isDone = `isDone`;""".}
  let modificationDate = newNSDate(todo.getModificationDate)
  {.emit: """self.modificationDate = `modificationDate`;""".}

proc getId*(self: NRTodo): int {.NoStackFrame, inline.} =
  ## Returns the identifier of the NRTodo object.
  {.emit: """return [self id_];""".}

proc text*(self: NRTodo): cString {.NoStackFrame, inline.} =
  ## Returns the text of the NRTodo object.
  {.emit: """return [[self text] UTF8String];""".}

proc priority*(self: NRTodo): int {.NoStackFrame, inline.} =
  ## Returns the priority of the NRTodo object.
  {.emit: """return [self priority];""".}

proc isDone*(self: NRTodo): bool {.NoStackFrame, inline.} =
  ## Returns the boolean done status of the NRTodo object.
  {.emit: """return [self isDone];""".}

proc sendNotification*[T](self: T; name: NSString) {.NoStackFrame, inline.} =
  ## Sends the specified notification with the current object as emitter.
  {.emit: """
    NSNotification *notification = [NSNotification
      notificationWithName:`name` object:self];
    [[NSNotificationQueue defaultQueue] enqueueNotification:notification
      postingStyle:NSPostASAP];
      """.}

proc update*(self: NRPagedParams; params: TPagedParams) =
  ## Updates a NRPagedParams object with values from its nimrod counterpart.
  var value = params.priorityAscending;
  {.emit: """self.priorityAscending = `value`;""".}
  value = params.dateAscending;
  {.emit: """self.dateAscending = `value`;""".}
  value = params.showUnchecked;
  {.emit: """self.showUnchecked = `value`;""".}
  value = params.showChecked;
  {.emit: """self.showChecked = `value`;""".}

proc priorityAscending*(self: NRPagedParams): bool {.NoStackFrame, inline.} =
  ## Returns the boolean priority status of the NRPagedParams object.
  {.emit: """return [self priorityAscending];""".}

proc dateAscending*(self: NRPagedParams): bool {.NoStackFrame, inline.} =
  ## Returns the boolean date status of the NRPagedParams object.
  {.emit: """return [self dateAscending];""".}

proc showUnchecked*(self: NRPagedParams): bool {.NoStackFrame, inline.} =
  ## Returns the boolean unchecked status of the NRPagedParams object.
  {.emit: """return [self showUnchecked];""".}

proc showChecked*(self: NRPagedParams): bool {.NoStackFrame, inline.} =
  ## Returns the boolean checked status of the NRPagedParams object.
  {.emit: """return [self showChecked];""".}
