# Implements the interface between objc and nimrod code.

import backend, db_sqlite, strutils, helpers, marshal

type
  TGlobalState = object       # Stores the global state for the app.
    conn: TDbConn             # Keeps around the database connection.
    didFetchOnce: bool        # Remembers if we have already queried SQLite.
    todos: seq[TTodo]         # Cache of items from the database.
    fetchParams: TPagedParams # Parameters for the next SQL fetch.

var g : TGlobalState

# Declare some global notification constants
var
  NRTodoAdded {.exportc.} = newNSString("NRTodoAdded")
  NRTodoModified {.exportc.} = newNSString("NRTodoModified")
  NRTodoDeleted {.exportc.} = newNSString("NRTodoDeleted")
  NRFetchOptionsDidChange {.exportc.} = newNSString("NRFetchOptionsDidChange")


proc echod*(s: varargs[string, `$`]) =
  ## Basic debug echo, goes away in release versions.
  when not defined(release):
    stdout.write("nimrod: ")
    for i in items(s): stdout.write(i)
    echo()

proc generateDatabaseRows(conn: TDbConn) =
  ## Adds some rows to the database ignoring errors.
  ##
  ## This proc was copied straight from the commandline example.
  discard conn.addTodo(1, "Watch another random youtube video")
  discard conn.addTodo(2, "Train some starcraft moves for the league")
  discard conn.addTodo(3, "Spread the word about Nimrod")
  discard conn.addTodo(4, "Give fruit superavit to neighbours")
  var todo = conn.addTodo(4, "Send tax form through snail mail")
  todo.isDone = true
  discard todo.save(conn)
  discard conn.addTodo(1, "Download new anime to watch")
  todo = conn.addTodo(2, "Build train model from scraps")
  todo.isDone = true
  discard todo.save(conn)
  discard conn.addTodo(5, "Buy latest Britney Spears album")
  discard conn.addTodo(6, "Learn a functional programming language")
  echo("Generated some entries, they were added to your database.")

proc addTodo(todo: NRTodo): bool {.cdecl, exportc.} =
  ## Adds an objc todo to the database.
  ##
  ## Returns true if the addition was without problems. In this case we don't
  ## refetch directly the global list, we mark the didFetchOnce as false to
  ## force a true sqlite fetch the next time a request is performed for new
  ## data.
  ##
  ## A successful addition of the todo object to the database will modify this
  ## object's id attribute and also generate the global notification
  ## NRTodoAdded with the todo as sender object.
  try:
    let newTodo = g.conn.addTodo(todo.priority, $todo.text)
    todo.update(newTodo)
    result = true
    g.didFetchOnce = false
    todo.sendNotification(NRTodoAdded)
  except EDb:
    return

proc deleteTodo(todo: NRTodo; generateNotification:bool):
                bool {.cdecl, exportc.} =
  ## Deletes the specified todo from the database.
  ##
  ## Returns true if the deletion was successful, in which case it is also
  ## removed from the cached list. Pass true as the generateNotification
  ## parameter if you want the NRTodoDeleted notification to be generated. You
  ## may not want to generate it if you are the controller of a list, who
  ## already is in charge of the deletion.
  let numDeleted = g.conn.deleteTodo(todo.getId)
  if numDeleted > 0:
    result = true
    # Search for the item and delete it.
    for i in 0..len(g.todos)-1:
      if g.todos[i].getId == todo.getId:
        g.todos.delete(i)
        if generateNotification:
          todo.sendNotification(NRTodoDeleted)
        break

proc fetchTodos(): NSMutableArray {.cdecl, exportc, discardable.} =
  ## Returns all the items in the database.
  ##
  ## Actually since we know on iOS there is no multitasking, we will fetch the
  ## items once and keep all of them in memory.
  if not g.didFetchOnce:
    g.didFetchOnce = true
    g.todos = g.conn.getPagedTodos(g.fetchParams)
    echod "Caching $1 todo objects" % $g.todos.len

  result = newNSMutableArray(g.todos.len)
  for todo in items(g.todos):
    var temp = newNRTodo()
    temp.update(todo)
    result.addObject(temp)

proc updateTodo(todo: NRTodo): bool {.cdecl, exportc.} =
  ## Searches for the todo in the list and updates it from the database.
  ##
  ## If the todo is not present, the update will fail and return false.
  if not g.didFetchOnce:
    fetchTodos()

  for i in 0..len(g.todos)-1:
    var cachedTodo = g.todos[i]
    if cachedTodo.getId == todo.getId:
      if cachedTodo.update(g.conn):
        todo.update(cachedTodo)
        result = true

proc saveTodo(todo: NRTodo): bool {.cdecl, exportc.} =
  ## Searches for the todo in the list and saves it to the database.
  ##
  ## If the todo is not present, the saving will fail and return false.
  ## A successful addition of the todo object to the database will modify this
  ## object's date attribute and also generate the global notification
  ## NRTodoModified with the todo as sender object.
  for i in 0..len(g.todos)-1:
    var cachedTodo = g.todos[i]
    if cachedTodo.getId == todo.getId:
      cachedTodo.text = $todo.text
      cachedTodo.priority = todo.priority
      cachedTodo.isDone = todo.isDone
      # If the saving goes well, update the objc version too.
      if cachedTodo.save(g.conn):
        g.didFetchOnce = false
        todo.update(cachedTodo)
        todo.sendNotification(NRTodoModified)
        result = true

proc initTodoInterface(filename: NSString, autogenerate:bool):
                       bool {.cdecl, exportc.} =
  ## Initialises the global variable holding the state for the todo interface.
  ## Pass the full path to the name of the sqlite database and true for
  ## autogenerate if you want the interface to create some random todo entries.
  ##
  ## Returns false if something went wrong.
  let path = $filename.getCString
  echod "initTodoInterface('$1')" % path
  try:
    g.conn = openDatabase(path)
    g.fetchParams.initDefaults
    if autogenerate:
      g.conn.generateDatabaseRows
    result = true
  except:
    nil

proc closeTodoInterface() {.cdecl, exportc.} =
  ## Called at program exit, tries to cleanly close any open resources.
  if g.conn != nil:
    g.conn.close()
    g.conn = nil

proc reopenTodoInterface(filename: NSString): bool {.cdecl, exportc.} =
  ## Reopens the global database connection with the specified file path.
  ##
  ## Returns false if something went really wrong.
  let path = $filename.getCString
  try:
    g.conn = openDatabase(path)
    result = true
  except:
    nil

proc updateFetchParams(params: NRPagedParams) {.cdecl, exportc.} =
  ## Updates the given structure with the global parameters.
  params.update(g.fetchParams)

proc saveFetchParams(params: NRPagedParams) {.cdecl, exportc.} =
  ## Saves the given structure as the new global parameters for fetches.
  ##
  ## This will also invalidate the current fetch list and generate a
  ## NRFetchOptionsDidChange notification.
  g.fetchParams.priorityAscending = params.priorityAscending
  g.fetchParams.dateAscending = params.dateAscending
  g.fetchParams.showUnchecked = params.showUnchecked
  g.fetchParams.showChecked = params.showChecked
  g.didFetchOnce = false
  params.sendNotification(NRFetchOptionsDidChange)

proc unmarshalFetchParams(data: NSString) {.cdecl, exportc.} =
  ## Sets the global parameters from the serialized NSString.
  g.fetchParams = to[TPagedParams]($data.getCString)

proc marshalFetchParams(): cstring {.cdecl, exportc.} =
  ## Returns the current fetch parameters as a marshalled cstring for saving.
  return cstring($$g.fetchParams)
