#import "NRListViewController.h"

#import "NRFetchOptionsController.h"
#import "NRTodo.h"
#import "NRTodoCell.h"
#import "NRTodoViewController.h"
#import "todointerface.h"

#import "ELHASO.h"


@interface NRListViewController ()

/// Holds the available todo items.
@property (nonatomic, retain) NSMutableArray *items;

/// Keeps a pointer to the last item the user wanted to edit.
@property (nonatomic, retain) NRTodo *childToModify;

@end


@implementation NRListViewController

@synthesize items = items_;
@synthesize childToModify = childToModify_;

#pragma mark -
#pragma mark Life

- (void)viewDidLoad
{
	[super viewDidLoad];
	self.view.frame = self.view.bounds;

	self.title = @"NimTodo";
	self.navigationItem.leftBarButtonItem = self.editButtonItem;

	UIBarButtonItem *button = [[[UIBarButtonItem alloc]
		initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
		target:self action:@selector(addTodo)] autorelease];
	self.navigationItem.rightBarButtonItem = button;

	button = [[UIBarButtonItem alloc]
		initWithTitle:@"Filter options" style:UIBarButtonItemStyleBordered
		target:self action:@selector(didTouchFilterOptions)];
	[self setToolbarItems:[NSArray arrayWithObject:button] animated:NO];

	[self.navigationController setToolbarHidden:NO animated:YES];

	self.items = fetchTodos();

	// Watch out for new too additions.
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(todosDidChange:) name:NRTodoAdded object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(todosDidChange:) name:NRTodoModified object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(todosDidChange:) name:NRTodoDeleted object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(todosDidChange:) name:NRFetchOptionsDidChange
		object:nil];
}

- (void)viewDidUnload
{
	self.items = nil;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter]
		removeObserver:self name:NRTodoAdded object:nil];
	[[NSNotificationCenter defaultCenter]
		removeObserver:self name:NRTodoModified object:nil];
	[[NSNotificationCenter defaultCenter]
		removeObserver:self name:NRTodoDeleted object:nil];
	[[NSNotificationCenter defaultCenter]
		removeObserver:self name:NRFetchOptionsDidChange object:nil];
	[childToModify_ release];
	[items_ release];
	[super dealloc];
}

/// When the list reappears, we surely have finished modifying any item.
- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	self.childToModify = nil;
}

#pragma mark -
#pragma mark Methods

- (BOOL)shouldAutorotateToInterfaceOrientation:
	(UIInterfaceOrientation)interfaceOrientation
{
	return YES;
}

/// Called when the user touches the add button.
- (void)addTodo
{
	NRTodoViewController *c = [[NRTodoViewController new] autorelease];
	c.creationMode = YES;
	UINavigationController *nav = [[[UINavigationController alloc]
		initWithRootViewController:c] autorelease];
	[self presentModalViewController:nav animated:YES];
}

/** Something in the database regarding todos changed.
 * We could do something clever, like add/modify/delete the todo directly to
 * our array, but we are lazy and simply reload all the entries. Also, if the
 * user entered the view of a todo item and change it, it is possible that a
 * new call to fetchTodos would return a list without it. This could happen if
 * the list is viewing only checked items, and the user checks it in the child
 * controller. So we have to rescan the new list, and if there is no item with
 * the same id as the last child todo, pop all view controllers.
 */
- (void)todosDidChange:(NSNotification*)notification
{
	DLOG(@"Todo %@ generated notification %@",
		notification.object, notification.name);
	self.items = fetchTodos();
	[self.tableView reloadData];

	if (self.childToModify) {
		// Scan the child in the current list of items.
		NRTodo *found = nil;
		for (NRTodo *item in self.items) {
			if (item.id_ == self.childToModify.id_) {
				found = item;
				break;
			}
		}

		if (!found) {
			// Ok, remove the item and pop all controllers.
			[self.navigationController popToViewController:self animated:YES];
			self.childToModify = nil;
		}
	}
}

/// The user wants to change the filtering options, show selector.
- (void)didTouchFilterOptions
{
	NRFetchOptionsController *c = [[NRFetchOptionsController new] autorelease];
	UINavigationController *nav = [[[UINavigationController alloc]
		initWithRootViewController:c] autorelease];
	[self presentModalViewController:nav animated:YES];
}

#pragma mark -
#pragma mark Table

/// Complies with the protocol, tells the table how many items it has.
- (NSInteger)tableView:(UITableView *)tableView
	numberOfRowsInSection:(NSInteger)section
{
	return self.items.count;
}

/// Builds the cell object for a specific index.
- (UITableViewCell *)tableView:(UITableView *)tableView
	cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString *cell_id = @"todoCell";

	// Dequeue or create a new cell.
	NRTodoCell *cell = (id)[tableView
		dequeueReusableCellWithIdentifier:cell_id];

	if (nil == cell) {
		cell = [[[NRTodoCell alloc]
			initWithStyle:UITableViewCellStyleSubtitle
			reuseIdentifier:cell_id] autorelease];
	}

	cell.todo = [self.items objectAtIndex:indexPath.row];
	return cell;
}

/// Handles deletion of cells in edition mode.
- (void)tableView:(UITableView *)tableView
	commitEditingStyle:(UITableViewCellEditingStyle)editingStyle
	forRowAtIndexPath:(NSIndexPath *)indexPath
{
	if (UITableViewCellEditingStyleDelete != editingStyle)
		return;

	// Deletes the object at the given index path.
	NRTodo *todo = [self.items objectAtIndex:indexPath.row];
	DLOG(@"Deleting %@", todo);
	deleteTodo(todo, false);

	// Update our local array and table view.
	[self.items removeObjectAtIndex:indexPath.row];
	[tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
		withRowAnimation:YES];
}

/// Handles touches on a cell, displaying the touched element.
- (void)tableView:(UITableView *)tableView
	didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	NRTodoViewController *c = [[NRTodoViewController new] autorelease];
	c.hidesBottomBarWhenPushed = YES;
	c.creationMode = NO;
	c.todo = [self.items objectAtIndex:indexPath.row];
	if (c.todo) {
		// Keep a pointer to the object the user wants to view/modify.
		self.childToModify = c.todo;
		[self.navigationController pushViewController:c animated:YES];
	}
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
}

@end
